@@ -20,12 +20,42 @@
 #include "pythonvariant.h"
 #include "pythonextension.h"
 
-//#include <kross/core/variant.h>
+#include <kross/core/manager.h>
+#include <kross/core/wrapperinterface.h>
 
 #include <QWidget>
 
 using namespace Kross;
 
+namespace Kross {
+
+    /// \internal helper class to deal with generic QList-types.
+    class VoidList : public QList<void*> {
+        public:
+            VoidList() : QList<void*>() {}
+            VoidList(QList<void*> list, const QByteArray& typeName) : QList<void*>(list), typeName(typeName) {}
+            QByteArray typeName;
+
+            static void* extractVoidStar(const Py::Object& object)
+            {
+                QVariant v = PythonType<QVariant>::toVariant(object);
+                if( QObject* obj = qVariantCanConvert< QWidget* >(v) ? qvariant_cast< QWidget* >(v) : qVariantCanConvert< QObject* >(v) ? qvariant_cast< QObject* >(v) : 0 ) {
+                    if( WrapperInterface* wrapper = dynamic_cast<WrapperInterface*>(obj) )
+                        return wrapper->wrappedObject();
+                    return obj;
+                }
+                if( void* ptr = v.value< void* >() ) {
+                    return ptr;
+                }
+                return 0; //TODO handle other cases?!
+            }
+
+    };
+
+}
+
+Q_DECLARE_METATYPE(Kross::VoidList)
+
 Py::Object PythonType<QVariant>::toPyObject(const QVariant& v)
 {
     #ifdef KROSS_PYTHON_VARIANT_DEBUG
@@ -102,6 +132,26 @@ Py::Object PythonType<QVariant>::toPyObject(const QVariant& v)
                 return PythonType<double>::toPyObject(v.toDouble());
             }
 
+            if( strcmp(v.typeName(),"Kross::VoidList") == 0 ) {
+                VoidList list = v.value<VoidList>();
+                Kross::MetaTypeHandler* handler = Kross::Manager::self().metaTypeHandler(list.typeName);
+                #ifdef KROSS_PYTHON_VARIANT_DEBUG
+                    krossdebug( QString("PythonType<QVariant>::toPyObject Casting '%1' to QList<%2> with %3 items, hasHandler=%4").arg(v.typeName()).arg(list.typeName.constData()).arg(list.count()).arg(handler ? "true" : "false") );
+                #endif
+                QVariantList l;
+                foreach(void* ptr, list) {
+                    if( handler ) {
+                        l << handler->callHandler(ptr);
+                    }
+                    else {
+                        QVariant v;
+                        v.setValue(ptr);
+                        l << v;
+                    }
+                }
+                return PythonType<QVariantList>::toPyObject(l);
+            }
+
             if( qVariantCanConvert< Kross::Object::Ptr >(v) ) {
                 #ifdef KROSS_PYTHON_VARIANT_DEBUG
                     krossdebug( QString("PythonType<QVariant>::toPyObject Casting '%1' to Kross::Object::Ptr").arg(v.typeName()) );
@@ -145,8 +195,13 @@ Py::Object PythonType<QVariant>::toPyObject(const QVariant& v)
                 return Py::asObject(new PythonExtension(obj));
             }
 
-            //QObject* obj = (*reinterpret_cast< QObject*(*)>( variantargs[0]->toVoidStar() ));
-            //PyObject* qobjectptr = PyLong_FromVoidPtr( (void*) variantargs[0]->toVoidStar() );
+            if( qVariantCanConvert< void* >(v) ) {
+                #ifdef KROSS_PYTHON_VARIANT_DEBUG
+                    krossdebug( QString("PythonType<QVariant>::toPyObject Casting '%1' to VoidStar").arg(v.typeName()) );
+                #endif
+                PyObject* voidptr = PyLong_FromVoidPtr( (void*) qvariant_cast<void*>(v) );
+                return Py::asObject( voidptr );
+            }
 
             //if(v.type() == QVariant::Invalid) return Py::None();
 
@@ -160,47 +215,36 @@ Py::Object PythonType<QVariant>::toPyObject(const QVariant& v)
 
 QVariant PythonType<QVariant>::toVariant(const Py::Object& obj)
 {
-    if(obj == Py::None()) {
-        #ifdef KROSS_PYTHON_VARIANT_DEBUG
-            krossdebug( QString("PythonType<QVariant>::toVariant Py::None") );
-        #endif
-        return QVariant();
-    }
-
-    PyTypeObject *type = (PyTypeObject*) obj.type().ptr();
     #ifdef KROSS_PYTHON_VARIANT_DEBUG
-        krossdebug( QString("PythonType<QVariant>::toVariant type=%1").arg(type->tp_name) );
+        krossdebug( QString("PythonType<QVariant>::toVariant str=%1").arg(obj.as_string().c_str()) );
     #endif
 
-    if(type == &PyInt_Type)
+    // check for None
+    if(obj == Py::None())
+        return QVariant();
+
+    PyObject* pyobj = obj.ptr();
+    Q_ASSERT(pyobj);
+
+    // check for some basic buildin types
+    if( PyObject_TypeCheck(pyobj, &PyInt_Type) )
         return PythonType<int>::toVariant(obj);
-    if(type == &PyLong_Type)
+    if( PyObject_TypeCheck(pyobj, &PyLong_Type) )
         return PythonType<qlonglong>::toVariant(obj);
-    if(type == &PyFloat_Type)
+    if( PyObject_TypeCheck(pyobj, &PyFloat_Type) )
         return PythonType<double>::toVariant(obj);
-    if(type == &PyBool_Type)
+    if( PyObject_TypeCheck(pyobj, &PyBool_Type) )
         return PythonType<bool>::toVariant(obj);
-    if(obj.isString())
+    if( obj.isString() )
         return PythonType<QString>::toVariant(obj);
-    if(type == &PyTuple_Type)
+    if( PyObject_TypeCheck(pyobj, &PyTuple_Type) )
         return PythonType<QVariantList,Py::Tuple>::toVariant(Py::Tuple(obj));
-    if(type == &PyList_Type)
+    if( PyObject_TypeCheck(pyobj, &PyList_Type) )
         return PythonType<QVariantList,Py::List>::toVariant(Py::List(obj));
-    if(type == &PyDict_Type)
+    if( PyObject_TypeCheck(pyobj, &PyDict_Type) )
         return PythonType<QVariantMap,Py::Dict>::toVariant(Py::Dict(obj.ptr()));
 
-    if(obj.isInstance()) {
-        #ifdef KROSS_PYTHON_VARIANT_DEBUG
-        krossdebug( QString("PythonType<QVariant>::toVariant IsInstance=TRUE") );
-        #endif
-        //return new PythonType(object);
-        QVariant result;
-        Kross::Object::Ptr p;
-        p.attach(new Kross::PythonObject(obj));
-        result.setValue(p);
-        return result;
-    }
-
+    // check if it's an extension
     if(PythonExtension::check(obj.ptr())) {
         Py::ExtensionObject<PythonExtension> extobj(obj);
         PythonExtension* extension = extobj.extensionObject();
@@ -221,8 +265,24 @@ QVariant PythonType<QVariant>::toVariant(const Py::Object& obj)
         return variant;
     }
 
-    throw Py::RuntimeError( QString("Invalid object of type '%1'.").arg(type->tp_name).toLatin1().constData() );
-    return QVariant();
+    // This is a bit messy. We are only interested in instances here but newer Python versions don't care
+    // any longer and don't really make any difference between buildin types and class-instances. To have
+    // at least some kind of safety we check for the Py_TPFLAGS_HEAPTYPE flag and if defined, just put
+    // the PyObject into a Kross::Object then.
+
+    //PyTypeObject *type = (PyTypeObject*) obj.type().ptr(); // can throw a Py::TypeError
+    //if( obj.isInstance() || PyClass_Check(pytype) || PyObject_TypeCheck(pyobj, &PyBaseObject_Type) ) {
+    //if( PyObject_TypeCheck(pyobj, &PyBaseObject_Type) ) {
+    //if(obj.ptr()->ob_type->tp_flags & Py_TPFLAGS_HEAPTYPE) {
+    #ifdef KROSS_PYTHON_VARIANT_DEBUG
+    krossdebug( QString("PythonType<QVariant>::toVariant convert to Kross::PythonObject") );
+    #endif
+    //return new PythonType(object);
+    QVariant result;
+    Kross::Object::Ptr p;
+    p.attach(new Kross::PythonObject(obj));
+    result.setValue(p);
+    return result;
 }
 
 #if 0
@@ -351,9 +411,12 @@ MetaType* PythonMetaTypeFactory::create(const char* typeName, const Py::Object&
 
                 Py::ExtensionObject<PythonExtension> extobj(object);
                 PythonExtension* extension = extobj.extensionObject();
-                if( ! extension->object() )
+                QObject* obj = extension->object();
+                if( ! obj )
                     throw Py::RuntimeError( QString("Underlying QObject instance of the PythonExtension was removed.").toLatin1().constData() );
-                return new MetaTypeVoidStar( typeId, extension->object(), owner );
+                if( WrapperInterface* wrapper = dynamic_cast<WrapperInterface*>(obj) )
+                    return new MetaTypeVoidStar( typeId, wrapper->wrappedObject(), owner );
+                return new MetaTypeVoidStar( typeId, obj, owner );
             }
 
             /*
@@ -433,6 +496,30 @@ MetaType* PythonMetaTypeFactory::create(const char* typeName, const Py::Object&
                 }
             }
 
+            // handle custom types within a QList by converting the list of pointers into a QList<void*>
+            QByteArray tn(typeName);
+            if( tn.startsWith("QList<") && tn.endsWith("*>") ) {
+                QByteArray itemTypeName = tn.mid(6, tn.length()-7);
+                #ifdef KROSS_PYTHON_VARIANT_DEBUG
+                    krosswarning( QString("PythonMetaTypeFactory::create Convert Py::Object '%1' to QList<void*> with typeName='%2' and itemTypeName='%3'").arg(object.as_string().c_str()).arg(typeName).arg(itemTypeName.constData()) );
+                #endif
+                QList<void*> list;
+                if( object.isTuple() ) {
+                    Py::Tuple t(object);
+                    for(uint i = 0; i < t.size(); ++i)
+                        if( void *ptr = VoidList::extractVoidStar(t[i]) )
+                            list << ptr;
+                }
+                else if( object.isList() ) {
+                    Py::List l(object);
+                    for(uint i = 0; i < l.length(); ++i)
+                        if( void *ptr = VoidList::extractVoidStar(l[i]) )
+                            list << ptr;
+                }
+                return new Kross::MetaTypeImpl< VoidList >(VoidList(list, itemTypeName));
+            }
+
+            // still no success. So, let's try to guess the content...
             QVariant v = PythonType<QVariant>::toVariant(object);
             if( qVariantCanConvert< Kross::Object::Ptr >(v) ) {
                 #ifdef KROSS_PYTHON_VARIANT_DEBUG
