@@ -1,6 +1,7 @@
 #include <ktexteditor/annotationinterface.h>
 #include <ktexteditor/attribute.h>
 #include <ktexteditor/codecompletioninterface.h>
+#include <ktexteditor/codecompletionmodelcontrollerinterface.h>
 #include <ktexteditor/codecompletionmodel.h>
 #include <ktexteditor/commandinterface.h>
 #include <ktexteditor/configinterface.h>
@@ -12,7 +13,9 @@
 #include <ktexteditor/editorchooser.h>
 #include <ktexteditor/editor.h>
 #include <ktexteditor/factory.h>
+#include <ktexteditor/highlightinterface.h>
 #include <ktexteditor/ktexteditor_export.h>
+#include <ktexteditor/loadsavefiltercheckplugin.h>
 #include <ktexteditor/markinterface.h>
 #include <ktexteditor/modificationinterface.h>
 #include <ktexteditor/plugin.h>
@@ -21,8 +24,12 @@
 #include <ktexteditor/searchinterface.h>
 #include <ktexteditor/sessionconfiginterface.h>
 #include <ktexteditor/smartcursor.h>
+#include <ktexteditor/smartcursornotifier.h>
+#include <ktexteditor/smartcursorwatcher.h>
 #include <ktexteditor/smartinterface.h>
 #include <ktexteditor/smartrange.h>
+#include <ktexteditor/smartrangenotifier.h>
+#include <ktexteditor/smartrangewatcher.h>
 #include <ktexteditor/templateinterface.h>
 #include <ktexteditor/texthintinterface.h>
 #include <ktexteditor/variableinterface.h>
